import React from 'react';
import { renderToString } from 'react-dom/server';
import { UseHighlighter } from './highlighter';
const DefaultIcons = {
    NOTE: (React.createElement("svg", { width: "24", height: "24", fill: "currentColor", viewBox: "0 0 24 24", className: "mr-2" },
        React.createElement("circle", { cx: "12", cy: "12", r: "10", stroke: "currentColor", strokeWidth: "2", fill: "none" }),
        React.createElement("text", { x: "12", y: "16", textAnchor: "middle", fontSize: "12", fontWeight: "bold", fill: "currentColor" }, "i"))),
    TIP: (React.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", viewBox: "0 0 24 24", width: "24", height: "24", fill: "currentColor", className: "mr-2" },
        React.createElement("path", { d: "M12 2.5c-3.81 0-6.5 2.743-6.5 6.119 0 1.536.632 2.572 1.425 3.56.172.215.347.422.527.635l.096.112c.21.25.427.508.63.774.404.531.783 1.128.995 1.834a.75.75 0 0 1-1.436.432c-.138-.46-.397-.89-.753-1.357a18.111 18.111 0 0 0-.582-.714l-.092-.11c-.18-.212-.37-.436-.555-.667C4.87 12.016 4 10.651 4 8.618 4 4.363 7.415 1 12 1s8 3.362 8 7.619c0 2.032-.87 3.397-1.755 4.5-.185.23-.375.454-.555.667l-.092.109c-.21.248-.405.481-.582.714-.356.467-.615.898-.753 1.357a.751.751 0 0 1-1.437-.432c.213-.706.592-1.303.997-1.834.202-.266.419-.524.63-.774l.095-.112c.18-.213.355-.42.527-.634.793-.99 1.425-2.025 1.425-3.561C18.5 5.243 15.81 2.5 12 2.5ZM8.75 18h6.5a.75.75 0 0 1 0 1.5h-6.5a.75.75 0 0 1 0-1.5Zm.75 3.75a.75.75 0 0 1 .75-.75h3.5a.75.75 0 0 1 0 1.5h-3.5a.75.75 0 0 1-.75-.75Z" }))),
    IMPORTANT: (React.createElement("svg", { width: "24", height: "24", fill: "currentColor", viewBox: "0 0 24 24", className: "mr-2" },
        React.createElement("rect", { width: "20", height: "16", x: "2", y: "4", rx: "2", stroke: "currentColor", strokeWidth: "2", fill: "none" }),
        React.createElement("text", { x: "12", y: "14", textAnchor: "middle", fontSize: "12", fontWeight: "bold", fill: "currentColor" }, "i"))),
    WARNING: (React.createElement("svg", { width: "24", height: "24", fill: "currentColor", viewBox: "0 0 24 24", className: "mr-2" },
        React.createElement("path", { d: "M12 2L2 22h20L12 2zm0 16h-2v-2h2v2zm0-4h-2v-4h2v4z" }))),
    CAUTION: (React.createElement("svg", { width: "24", height: "24", fill: "currentColor", viewBox: "0 0 24 24", className: "mr-2" },
        React.createElement("circle", { cx: "12", cy: "12", r: "10", stroke: "currentColor", strokeWidth: "2", fill: "none" }),
        React.createElement("text", { x: "12", y: "16", textAnchor: "middle", fontSize: "12", fontWeight: "bold", fill: "currentColor" }, "!")))
};
const DefaultalertTypes = {
    NOTE: 'bg-blue-100 border-blue-500 text-blue-700',
    TIP: 'bg-green-100 border-green-500 text-green-700',
    IMPORTANT: 'bg-yellow-100 border-yellow-500 text-yellow-700',
    WARNING: 'bg-orange-100 border-orange-500 text-orange-700',
    CAUTION: 'bg-red-100 border-red-500 text-red-700',
    DEFAULT: 'bg-gray-100 border-gray-500 text-gray-700'
};
function SimpleMarkdown({ content, className = 'dark:prose-invert', ctexTclass = "my-2", imageHeight = 400, theme = 'light', codeBlockTheme = "bg-[#fdf6e3] dark:bg-[#2d353b]", tableHeaderClass = "bg-gray-200 dark:bg-gray-700", tableCellClass = "border px-4 py-2", extraAlerts = [] }) {
    const icons = { ...DefaultIcons };
    const alertTypes = { ...DefaultalertTypes };
    //add extra alerts
    extraAlerts.forEach(alert => {
        icons[alert.ID] = alert.SVG;
        alertTypes[alert.ID] = alert.CLASSNAME;
    });
    const parseMarkdown = (md) => {
        const lines = md.split('\n');
        let inCodeBlock = false;
        let html = '';
        let currentParagraph = '';
        let inList = false;
        let listType = '';
        let currentCodeLanguage = '';
        let inTable = false;
        let tableHeader = '';
        let tableBody = '';
        let inBlockQuote = false;
        let blockQuoteContent = '';
        let blockQuoteType = '';
        const flushParagraph = () => {
            if (currentParagraph) {
                html += `<p class="${ctexTclass}">${currentParagraph.trim()}</p>`;
                //add \n
                currentParagraph = '';
            }
        };
        const flushList = () => {
            if (inList) {
                html += `</${listType}>`;
                inList = false;
                listType = '';
            }
        };
        const flushTable = () => {
            if (inTable) {
                html += `<table class="border-collapse table-auto w-full text-sm my-4">
            <thead class="${tableHeaderClass}">${tableHeader}</thead>
            <tbody>${tableBody}</tbody>
          </table>`;
                inTable = false;
                tableHeader = '';
                tableBody = '';
            }
        };
        const flushBlockQuote = () => {
            if (inBlockQuote) {
                const alertClass = alertTypes[blockQuoteType] || alertTypes.DEFAULT;
                html += `<blockquote class="border-l-4 pl-4 py-2 my-4 ${alertClass}">${blockQuoteContent}</blockquote>`;
                //replace the [!alertType] with the corresponding icon
                //return `<div class="flex items-center">${renderToString(icons[type as keyof typeof icons])}<span class="ml-2">${match.slice(2)}</span></div>`
                for (const type in icons) {
                    html = html.replace(`[!${type.toUpperCase()}]`, `<div class="flex items
                -center">${renderToString(icons[type])}<span class="ml-2">`);
                }
                //add  line separator
                inBlockQuote = false;
                blockQuoteContent = '';
                blockQuoteType = '';
            }
        };
        const parseInline = (text) => {
            // Bold
            text = text.replace(/\*\*(.*?)\*\*/g, '<strong>$1</strong>');
            // Italic
            text = text.replace(/\*(.*?)\*/g, '<em>$1</em>');
            // Inline code
            text = text.replace(/`([^`]+)`/g, '<code class="bg-gray-100 dark:bg-gray-800 px-1 rounded">$1</code>');
            // Iframe
            text = text.replace(/<iframe.*?src=["']([^"']+)["'].*?><\/iframe>/g, `<iframe width="100%" height="${imageHeight}" src="$1" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>`);
            // Links with title (alias)
            text = text.replace(/\[([^\]]+)\]$$([^)]+)\s*"?(.*?)"?$$/g, '<a href="$2" title="$3" target="_blank" rel="noopener noreferrer" class="text-blue-500 hover:underline">$1</a>');
            // Links without title
            text = text.replace(/\[([^\]]+)\]$$([^)]+)$$/g, '<a href="$2" target="_blank" rel="noopener noreferrer" class="text-blue-500 hover:underline">$1</a>');
            return text;
        };
        const highlightCode = (code, language) => {
            const codeComponent = UseHighlighter({ content: code, lang: language, theme: theme });
            return `${codeComponent}`;
        };
        const parseGitHubAlert = (line) => {
            const match = line.match(/>\s*\[!(NOTE|TIP|IMPORTANT|WARNING|CAUTION)\]/);
            if (match) {
                return match[1];
            }
            return null;
        };
        let codeLines = [];
        lines.forEach((line, index) => {
            if (line.trim().startsWith('```')) {
                flushParagraph();
                flushList();
                flushTable();
                flushBlockQuote();
                inCodeBlock = !inCodeBlock;
                if (inCodeBlock) {
                    currentCodeLanguage = line.trim().slice(3).toLowerCase() || 'default';
                    html += `<pre><code class="language-${currentCodeLanguage} ${codeBlockTheme} block p-2 rounded-md my-2 overflow-x-auto">`;
                    codeLines = [];
                }
                else {
                    html += '</code></pre>';
                    currentCodeLanguage = '';
                }
            }
            else if (inCodeBlock) {
                html += highlightCode(line, currentCodeLanguage).replace(`<code`, `<span class="text-gray-400 dark:text-gray-500 text-sm">${codeLines.length + 1} | </span><code`);
                codeLines.push(line);
            }
            else if (line.trim().startsWith('|') && line.includes('|')) {
                flushParagraph();
                flushList();
                flushBlockQuote();
                if (!inTable) {
                    inTable = true;
                    if (line.includes('---')) {
                        return;
                    }
                }
                const cells = line.split('|').slice(1, -1).map(cell => cell.trim());
                const row = cells.map(cell => `<td class="${tableCellClass}">${parseInline(cell)}</td>`).join('');
                if (tableHeader === '') {
                    tableHeader = `<tr>${row}</tr>`;
                }
                else {
                    tableBody += `<tr>${row}</tr>`;
                }
            }
            else if (line.trim().startsWith('>')) {
                flushParagraph();
                flushList();
                flushTable();
                if (!inBlockQuote) {
                    inBlockQuote = true;
                    blockQuoteType = parseGitHubAlert(line) || 'DEFAULT';
                }
                blockQuoteContent += parseInline(line.slice(1).trim()) + ' ';
                if (!line.includes('[!')) {
                    blockQuoteContent += '<br>';
                }
            }
            else {
                flushTable();
                flushBlockQuote();
                // Headers
                if (line.startsWith('#')) {
                    flushParagraph();
                    flushList();
                    const levelMatch = line.match(/^#+/);
                    if (levelMatch) {
                        const level = levelMatch[0].length;
                        const text = line.replace(/^#+\s*/, '');
                        html += `<h${level} class="text-${4 - level}xl font-bold mt-${8 - level} mb-${6 - level}">${parseInline(text)}</h${level}>`;
                    }
                }
                // Lists
                else if (/^\s*(\d+\.|-)\s+(.*)/.test(line)) {
                    flushParagraph();
                    const match = line.match(/^\s*(\d+\.|-)\s+(.*)/);
                    if (match) {
                        const isOrdered = match[1].endsWith('.');
                        if (!inList || (isOrdered && listType === 'ul') || (!isOrdered && listType === 'ol')) {
                            flushList();
                            listType = isOrdered ? 'ol' : 'ul';
                            html += `<${listType} class="list-${isOrdered ? 'decimal' : 'disc'} pl-5 my-2">`;
                            inList = true;
                        }
                        html += `<li>${parseInline(match[2])}</li>`;
                    }
                }
                // Images
                else if (/!\[([^\]]*)\]$$([^)]+)(?:\s+(\d+)x(\d+))?$$/.test(line)) {
                    flushParagraph();
                    line = line.replace(/!\[([^\]]*)\]$$([^)]+)(?:\s+(\d+)x(\d+))?$$/g, (match, alt, src, width, height) => {
                        const sizeAttr = width && height ? `width="${width}" height="${height}"` : '';
                        return `<img src="${src}" alt="${alt}" ${sizeAttr} class="my-2 rounded-md max-w-full">`;
                    });
                    html += line;
                }
                // Horizontal Rule
                else if (/^(-{3,}|\*{3,})$/.test(line.trim())) {
                    flushParagraph();
                    flushList();
                    html += '<hr class="my-4 border-t border-gray-300 dark:border-gray-700">';
                }
                // Paragraphs
                else {
                    if (line.trim() === '') {
                        flushParagraph();
                    }
                    else {
                        currentParagraph += parseInline(line) + ' ';
                    }
                }
            }
        });
        flushParagraph();
        flushList();
        flushTable();
        flushBlockQuote();
        return html;
    };
    return (React.createElement("div", { className: `prose ${className}`, dangerouslySetInnerHTML: { __html: parseMarkdown(content) } }));
}
export { SimpleMarkdown };
